#include <IMFFlow.h>
#include <IMFExpansion.h>
#include <IMFExpansionSpace.h>
#include <MFAtlas.h>
#include <stdlib.h>
#include <math.h>
#ifdef DMALLOC
#include <dmalloc.h>
#endif

double *MFNV_CStar(MFNVector);
double *MFKV_CStar(MFKVector);
double *MFNKM_CStar(MFNKMatrix);

struct IMFFlowSt {int n;
                  void (*F)(double*,double*);
                  void (*dF)(double*,double*);
                  void (*ddF)(double*, double*);
                  void (*dddF)(double*, double*);
                  int nRefs;
                  int fat;
                  IMFFlow fatF;
                  int k;
                  int dir;
                 };

void IMFIntegratorDTimeDerivatives(IMFFlow,double*,double*);

IMFFlow IMFCreateFlow(int n, void (*F)(double*,double*),
                             void (*dF)(double*,double*),
                             void (*ddF)(double*,double*),
                             void (*dddF)(double*,double*))
 {
  IMFFlow result;

  result=(IMFFlow)malloc(sizeof(struct IMFFlowSt));
  if(result==(IMFFlow)NULL)abort();

  result->n=n;

  result->F=F;
  result->dF=dF;
  result->ddF=ddF;
  result->dddF=dddF;
  result->dir=1;

  result->fat=0;
  result->fatF=(IMFFlow)NULL;
  result->k=0;
  result->nRefs=1;

  return result;
 }

IMFFlow IMFCreateFatFlow(IMFFlow F,int k)
 {
  IMFFlow result;
  int n,m;

  m=IMFFlowN(F);
  n=m+m*k+m*k*k;

  result=(IMFFlow)malloc(sizeof(struct IMFFlowSt));
  if(result==(IMFFlow)NULL)abort();

  result->n=n;

  result->F=(void (*)(double*,double*))NULL;
  result->dF=(void (*)(double*,double*))NULL;
  result->ddF=(void (*)(double*,double*))NULL;
  result->dddF=(void (*)(double*,double*))NULL;
  result->dir=1;

  result->fat=1;
  result->fatF=F;
  IMFRefFlow(F);
  result->k=k;

  result->nRefs=1;

  return result;
 }

IMFFlow IMFCreateBackwardFatFlow(IMFFlow F,int k)
 {
  IMFFlow result;
  int n,m;

  m=IMFFlowN(F);
  n=m+m*k+m*k*k;

  result=(IMFFlow)malloc(sizeof(struct IMFFlowSt));
  if(result==(IMFFlow)NULL)abort();

  result->n=n;

  result->F=(void (*)(double*,double*))NULL;
  result->dF=(void (*)(double*,double*))NULL;
  result->ddF=(void (*)(double*,double*))NULL;
  result->dddF=(void (*)(double*,double*))NULL;
  result->dir=-1;

  result->fat=1;
  result->fatF=F;
  IMFRefFlow(F);
  result->k=k;

  result->nRefs=1;

  return result;
 }

void IMFFreeFlow(IMFFlow F)
 {
  F->nRefs--;

  if(F->nRefs<=0)
   {
    if(!F->fat)
     {
      F->n=0;
      F->F=(void (*)(double*,double*))NULL;
      F->dF=(void (*)(double*,double*))NULL;
      F->ddF=(void (*)(double*,double*))NULL;
      F->dddF=(void (*)(double*,double*))NULL;
      F->fat=0;
      F->fatF=(IMFFlow)NULL;
      F->k=0;
     }else{
      F->n=0;
      F->F=(void (*)(double*,double*))NULL;
      F->dF=(void (*)(double*,double*))NULL;
      F->ddF=(void (*)(double*,double*))NULL;
      F->dddF=(void (*)(double*,double*))NULL;
      F->fat=0;
      IMFFreeFlow(F->fatF);
      F->fatF=(IMFFlow)NULL;
      F->k=0;
     }

    free(F);
   }

  return;
 }

void IMFRefFlow(IMFFlow F)
 {
  F->nRefs++;
  return;
 }

void IMFEvaluateFlow(IMFFlow F, MFNVector vu, double *f)
 {
  double *u;
  int i,n;

  if(!strcmp(MFNVGetId(vu),"DENSE"))
   {
    u=MFNV_CStar(vu);
   }else if(!strcmp(MFNVGetId(vu),"IMFExpansionVector"))
   {
    u=IMFExpansionU(IMFExpansionNVGetE(vu));
   }else abort();

  if(!F->fat)
   {
    F->F(u,f);
   }else{
    int k,m;
    int j,l,p,q,r,v;
    double d;
    double uFuu;
    static double *uFu=(double*)NULL;
    static double *sdf=(double*)NULL;
    static double *sddf=(double*)NULL;

    n=(F->fatF)->n;
    k=F->k;
    m=n+n*k+n*k*k;

    sdf=(double*)realloc((void*)sdf,n*n*sizeof(double));if(sdf==(double*)NULL)abort();
    sddf=(double*)realloc((void*)sddf,n*n*n*sizeof(double));if(sddf==(double*)NULL)abort();
    uFu=(double*)realloc((void*)uFu,k*k*sizeof(double));if(uFu==(double*)NULL)abort();

    IMFEvaluateFlow(F->fatF,vu,f);
    IMFEvaluateDerivativeOfFlow(F->fatF,vu,sdf);
    IMFEvaluateSecondDerivativeOfFlow(F->fatF,vu,sddf);

    d=0.;for(i=0;i<n;i++)d+=f[i]*f[i];d=sqrt(d+1.);
    for(i=0;i<n;i++)f[i]=f[i]/d;

    for(r=0;r<k;r++)
     for(j=0;j<k;j++)
      {
       uFu[r+k*j]=0.;
       for(p=0;p<n;p++)
         for(q=0;q<n;q++)
           uFu[r+k*j]+=u[n+p+n*r]*sdf[p+n*q]*u[n+q+n*j];
      }

    for(i=0;i<n*k;i++)f[n+i]=0.;

    for(j=0;j<k;j++)
     {
      for(i=0;i<n;i++)
       {
        for(p=0;p<n;p++)
          f[n+i+n*j]+=sdf[i+n*p]*u[n+p+n*j];

        for(r=0;r<k;r++)
          f[n+i+n*j]-=uFu[r+k*j]*u[n+i+n*r];

        f[n+i+n*j]=f[n+i+n*j]/d;
       }
     }

    for(i=0;i<n*k*k;i++)f[n+n*k+i]=0.;
#ifdef DOSECND
    for(l=0;l<k;l++)
     {
      for(j=0;j<k;j++)
       {
        for(i=0;i<n;i++)
         {
          for(p=0;p<n;p++)
            f[n+n*k+i+n*(j+k*l)]+=sdf[i+n*p]*u[n+n*k+p+n*(j+k*l)];
          for(p=0;p<n;p++)
            for(q=0;q<n;q++)
              f[n+n*k+i+n*(j+k*l)]+=sddf[i+n*(p+n*q)]*u[n+p+n*j]*u[n+q+n*l];
          for(r=0;r<k;r++)
            f[n+n*k+i+n*(j+k*l)]-=uFu[r+k*j]*u[n+n*k+i+n*(r+k*l)]+uFu[r+k*l]*u[n+n*k+i+n*(r+k*j)];

          for(r=0;r<k;r++)
           {
            uFuu=0.;
            for(p=0;p<n;p++)
              for(q=0;q<n;q++)
               {
                uFuu+=u[n+p+n*r]*sdf[p+n*q]*u[n+n*k+q+n*(j+k*l)];
                uFuu+=u[n+p+n*r]*sdf[q+n*p]*u[n+n*k+q+n*(j+k*l)];
                for(v=0;v<n;v++)
                  uFuu+=u[n+p+n*r]*sddf[p+n*(q+n*v)]*u[n+q+n*j]*u[n+v+n*l];
               }
            f[n+n*k+i+n*(j+k*l)]-=uFuu*u[i+n*r];
           }

          f[n+n*k+i+n*(j+k*l)]=1.*f[n+n*k+i+n*(j+k*l)]/d;
         }
       }
     }
#endif
   }

  n=F->n;
  if(F->dir==-1)for(i=0;i<n;i++)f[i]=-f[i];

  return;
 }

void IMFEvaluateDerivativeOfFlow(IMFFlow F, MFNVector vu, double *df)
 {
  double *u;
  int i,n;

  if(!strcmp(MFNVGetId(vu),"DENSE"))
   {
    u=MFNV_CStar(vu);
   }else if(!strcmp(MFNVGetId(vu),"IMFExpansionVector"))
   {
    u=IMFExpansionU(IMFExpansionNVGetE(vu));
   }else abort();

  if(!F->fat)
   {
    F->dF(u,df);
   }else if(1)
   {
    IMFIntegratorDTimeDerivatives(F,u,df);
   }else{
    int i,j,l,n,k,m;
    static double *sdf=(double*)NULL;
    static double *sf=(double*)NULL;
    double d;

    n=(F->fatF)->n;
    k=(F->fatF)->k;
    m=n+n*k+n*k*k;

    sdf=(double*)realloc((void*)sdf,n*n*sizeof(double));if(sdf==(double*)NULL)abort();
    sf=(double*)realloc((void*)sf,n*sizeof(double));if(sf==(double*)NULL)abort();

    IMFEvaluateFlow(F->fatF,vu,sf);
    IMFEvaluateDerivativeOfFlow(F->fatF,vu,sdf);

    d=0.;for(i=0;i<n;i++)d+=sf[i]*sf[i];d=sqrt(d);

    for(i=0;i<m*m;i++)df[i]=0.;
    for(i=0;i<n;i++)
      for(j=0;j<n;j++)df[i+m*j]=sdf[i+n*j]/d;
    for(i=0;i<n;i++)
      for(j=0;j<n;j++)
        for(l=0;l<n;l++)
          df[i+m*j]-=sf[i]*sf[l]*sdf[l+n*j]/d/d/d;
   }

  n=F->n;
  if(F->dir==-1)for(i=0;i<n*n;i++)df[i]=-df[i];

  return;
 }

void IMFEvaluateSecondDerivativeOfFlow(IMFFlow F, MFNVector vu, double *ddf)
 {
  double *u;
  int i,n;

  if(!strcmp(MFNVGetId(vu),"DENSE"))
   {
    u=MFNV_CStar(vu);
   }else if(!strcmp(MFNVGetId(vu),"IMFExpansionVector"))
   {
    u=IMFExpansionU(IMFExpansionNVGetE(vu));
   }else abort();

  if(!F->fat)
   {
    F->ddF(u,ddf);
   }else abort();

  n=F->n;
  if(F->dir==-1)for(i=0;i<n*n*n;i++)ddf[i]=-ddf[i];

  return;
 }

void IMFEvaluateThirdDerivativeOfFlow(IMFFlow F, MFNVector vu, double *dddf)
 {
  double *u;
  int i,n;

  if(!strcmp(MFNVGetId(vu),"DENSE"))
   {
    u=MFNV_CStar(vu);
   }else if(!strcmp(MFNVGetId(vu),"IMFExpansionVector"))
   {
    u=IMFExpansionU(IMFExpansionNVGetE(vu));
   }else abort();

  if(!F->fat)
   {
    F->dddF(u,dddf);
   }else abort();

  n=F->n;
  if(F->dir==-1)for(i=0;i<n*n*n*n;i++)dddf[i]=-dddf[i];

  return;
 }


int IMFFlowN(IMFFlow F)
 {
  return F->n;
 }

void IMFIntegratorDTimeDerivatives(IMFFlow F, double *y,double *dY)
 {
  double ysave;
  int i,j;
  int n;
  double eps=1.e-3;
  static double *ydotp=(double*)NULL;
  static double *ydotm=(double*)NULL;
  MFNVector vy;

  n=IMFFlowN(F);

  vy=MFCreateWrappedNVector(n,y);
  ydotp=(double*)realloc((void*)ydotp,n*sizeof(double));if(ydotp==(double*)NULL)abort();
  ydotm=(double*)realloc((void*)ydotm,n*sizeof(double));if(ydotm==(double*)NULL)abort();

  for(i=0;i<n;i++)
   {
    ysave=y[i];
    y[i]=ysave+eps;
    IMFEvaluateFlow(F,vy,ydotp);
    y[i]=ysave-eps;
    IMFEvaluateFlow(F,vy,ydotm);
    for(j=0;j<n;j++)dY[j+n*i]=.5*(ydotp[j]-ydotm[j])/eps;
    y[i]=ysave;
   }
  MFFreeNVector(vy);
  return;
 }

IMFFlow IMFCreateBackwardFlow(IMFFlow F)
 {
  IMFFlow result;

  result=IMFCreateFlow(F->n,F->F,F->dF,F->ddF,F->dddF);
  result->dir=-1;

  return result;
 }

double IMFFlowR(IMFFlow F, double eps, MFNVector u, MFNKMatrix mPhi)
 {
  double *phi;
  double fNorm;
  double R;
  double t;
  int i,j,l,m;
  int n,k;
  static double *dF=(double*)NULL;

/* Want to choose R so that |F(u)-F(u+R*Phi s)|<eps  */
/*                                  | R*Fu Phi s |<eps  */

  n=F->n;
  k=MFNKMatrixK(mPhi);

  phi=MFNKM_CStar(mPhi);
  dF=(double*)realloc((void*)dF,n*n*sizeof(double));if(dF==(double*)NULL)abort();
  IMFEvaluateFlow(F,u,dF);
  fNorm=0.;for(i=0;i<n;i++)fNorm+=dF[i]*dF[i];fNorm=sqrt(fNorm);
  IMFEvaluateDerivativeOfFlow(F,u,dF);
  R=0;
  for(i=0;i<k;i++)
   {
    for(j=0;j<k;j++)
     {
      t=0.;
      for(l=0;l<n;l++)
        for(m=0;m<n;m++)t+=phi[l+n*i]*dF[l+n*m]*phi[m+n*j];
      t=t/fNorm;
      if(i==0&&j==0||t>R)R=t;
     }
   }
  R=eps/fabs(R);
/*if(R<.3)R=.3;*/
  
  return R;
 }
