/* 
    @(#)MFCircle.c	1.12
    03/07/11 17:06:41
   
    PROGRAM NAME:  Manifold

    (c) COPYRIGHT INTERNATIONAL BUSINESS MACHINES
    CORPORATION 12/1/2001.  ALL RIGHTS RESERVED.

    Please refer to the LICENSE file in the top directory

*/
/*      author: Mike Henderson mhender@watson.ibm.com */
/*      date:   August 26, 1999                     */

#include <MFImplicitMF.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

double *MFKV_CStar(MFKVector);
double *MFNV_CStar(MFNVector);
double *MFNKM_CStar(MFNKMatrix);

extern double MFEpsilon;

static char MFCircleMFErrorMsg[256]="";
void MFSetError(int,char*,char*,int,char*);

void MFFreeCircleData(void*);
int MFProjectCircle(int,int,MFNVector,MFNKMatrix,MFNVector,void*,int*);
int MFTangentCircle(int,int,MFNVector,MFNKMatrix,void*);
double MFScaleCircle(int,int,MFNVector,MFNKMatrix,void*);
void MFWriteCircleData(FILE*,void*);
MFImplicitMF MFReadCircle(FILE*);

int MFCircleProjectToSave(MFNVector,double*,void*);
int MFCircleProjectToDraw(MFNVector,double*,void*);
int MFCircleProjectForBB(MFNVector,double*,void*);

MFImplicitMF MFIMFCreateCircle(double x,double y,double R)
 {
  static char RoutineName[]={"MFIMFCreateCircle"};
  MFImplicitMF circle;
  MFNSpace space;
  double *data;

  circle=MFIMFCreateBaseClass(2,1,"Circle");

  space=MFCreateNSpace(2);
  MFIMFSetSpace(circle,space);
  MFFreeNSpace(space);

  data=(double*)malloc(3*sizeof(double)); /*done*/
  if(data==(double*)NULL)
   {
    sprintf(MFCircleMFErrorMsg,"Out of memory, trying to allocate %d bytes",3*sizeof(double));
    printf("%s -- %s\n",RoutineName,MFCircleMFErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFCircleMFErrorMsg,__LINE__,__FILE__);
    free(circle);
    return (MFImplicitMF)NULL;
   }
  data[0]=x;
  data[1]=y;
  data[2]=R;
  MFIMFSetData(circle,data);
  MFIMFSetFreeData(circle,MFFreeCircleData);
  MFIMFSetProject(circle,MFProjectCircle);
  MFIMFSetTangent(circle,MFTangentCircle);
  MFIMFSetScale(circle,MFScaleCircle);
  MFIMFSetWriteData(circle,MFWriteCircleData);
  MFIMFSetProjectForSave(circle,MFCircleProjectToSave);
  MFIMFSetProjectForDraw(circle,MFCircleProjectToDraw);
  MFIMFSetProjectForBB(circle,MFCircleProjectForBB);

  return circle;
 }

void MFFreeCircleData(void *d)
 {
  static char RoutineName[]={"MFFreeCircleData"};
  free(d);
  return;
 }

int MFProjectCircle(int n,int k,MFNVector vu0,MFNKMatrix mPhi,MFNVector vu,void *d,int *index)
 {
  static char RoutineName[]={"MFProjectCircle"};
  double t,A,B,C,R,nx,ny;
  double u0x,u0y;
  double v0x,v0y;
  double err;

  double *u, *Phi, *u0;

  u0=MFNV_CStar(vu0);
  Phi=MFNKM_CStar(mPhi);
  u=MFNV_CStar(vu);

  v0x=Phi[0];
  v0y=Phi[1];
  nx= v0y;
  ny=-v0x;
  u0x=u0[0]-((double*)d)[0];
  u0y=u0[1]-((double*)d)[1];
  R=((double*)d)[2];

  A=nx*nx+ny*ny;
  B=nx*u0x+ny*u0y;
  C=u0x*u0x+u0y*u0y-R*R;
  if(A>0.)
   {
    if(B>0)
      t=(-B+sqrt(B*B-A*C))/A;
     else
      t=(-B-sqrt(B*B-A*C))/A;
   }else if(A==0.)
     t=-.5*C/B;
    else return 0;

  u[0]=u0[0]+t*nx;
  u[1]=u0[1]+t*ny;

  *index=0;

  return 1;
 }

int MFTangentCircle(int n,int k,MFNVector vu,MFNKMatrix mPhi,void *d)
 {
  static char RoutineName[]={"MFTangentCircle"};
  double s;
  double u0x,u0y;
  double v0x,v0y;
  double v1x,v1y;
  double err;
  double *u, *Phi;

  Phi=MFNKM_CStar(mPhi);
  u=MFNV_CStar(vu);

  u0x=(u[0]-((double*)d)[0])/((double*)d)[2];
  u0y=(u[1]-((double*)d)[1])/((double*)d)[2];

  v0x=-u0y;
  v0y= u0x;

  Phi[0]=-u0y;
  Phi[1]= u0x;

  return 1;
 }

double MFScaleCircle(int n,int k,MFNVector vu,MFNKMatrix mPhi,void *d)
 {
  static char RoutineName[]={"MFScaleCircle"};
  return sqrt(.5*MFEpsilon*((double*)d)[2]);
 }

void MFWriteCircleData(FILE *fid,void *d)
 {
  static char RoutineName[]={"MFWriteCircleData"};
  static double *data;

  data=(double*)d;

  fprintf(fid,"%lf %lf %lf\n",data[0],data[1],data[2]);
  fflush(fid);
  return;
 }

MFImplicitMF MFReadCircle(FILE *fid)
 {
  static char RoutineName[]={"MFReadCircle"};
  MFImplicitMF circle;
  double x=0.;
  double y=0.;
  double R=0.;

  fscanf(fid,"%lf %lf %lf\n",&x,&y,&R);
  circle=MFIMFCreateCircle(x,y,R);

  return circle;
 }

int MFCircleProjectToSave(MFNVector u, double *x, void *d)
 {
  if(x==(double*)NULL)return 2;

  x[0]=MFNV_C(u,0);
  x[1]=MFNV_C(u,1);

  return 0;
 }

int MFCircleProjectToDraw(MFNVector u, double *x, void *d)
 {
  if(x==(double*)NULL)return 2;

  x[0]=MFNV_C(u,0);
  x[1]=MFNV_C(u,1);

  return 0;
 }

int MFCircleProjectForBB(MFNVector u, double *x, void *d)
 {
  if(x==(double*)NULL)return 2;

  x[0]=MFNV_C(u,0);
  x[1]=MFNV_C(u,1);

  return 0;
 }
