/* 
    %W%
    %D% %T%
*/
/*      author: Mike Henderson mhender@watson.ibm.com */
/*      date:   March 1, 1999                         */

#include <MFNSpace.h>
#include <MFNVector.h>
#include <MFError.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <MFTime.h>

double *MFNV_CStar(MFNVector);

void MFSetError(int,char*,char*,int,char*);
static char MFTPBVPNSpaceErrorMsg[256]="";

struct MFTPBVPNSpaceData
 {
  int n;
  int nx;
  int nu;
  int np;
  int *parmPeriodic;
  double *parmPeriod;
 };

void MFFreeTPBVPNSpaceData(void*);
double MFTPBVPNSpaceDistance(MFNSpace,MFNVector,MFNVector,void*);
void MFTPBVPNSpaceDirection(MFNSpace,MFNVector,MFNVector,MFNVector,void*);
void MFTPBVPNSpaceAdd(MFNSpace,MFNVector,MFNVector,MFNVector,void*);
void MFTPBVPNSpaceScale(MFNSpace,double,MFNVector,MFNVector,void*);
double MFTPBVPNSpaceInner(MFNSpace,MFNVector,MFNVector,void*);
void MFWriteTPBVPNSpaceData(FILE*,MFNSpace,void*);
void MFReadTPBVPNSpaceData(FILE*,MFNSpace);

MFNSpace MFCreateTPBVPNSpace(int nx, int nu, int np)
 {
  static char RoutineName[]={"MFCreateTPBVPNSpace"};
  MFNSpace this;
  struct MFTPBVPNSpaceData *data;
  int i;

  this=MFCreateNSpaceBaseClass("TPBVPNSpace");
  MFNSpaceSetDistance(this,MFTPBVPNSpaceDistance);
  MFNSpaceSetInnerProduct(this,MFTPBVPNSpaceInner);
  MFNSpaceSetDirection(this,MFTPBVPNSpaceDirection);
  MFNSpaceSetAdd(this,MFTPBVPNSpaceAdd);
  MFNSpaceSetScale(this,MFTPBVPNSpaceScale);
  MFNSpaceSetFreeData(this,MFFreeTPBVPNSpaceData);
  MFNSpaceSetWriteData(this,MFWriteTPBVPNSpaceData);

  data=(struct MFTPBVPNSpaceData*)malloc(sizeof(struct MFTPBVPNSpaceData));
  if(data==(struct MFTPBVPNSpaceData*)NULL)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"Out of memory, trying to allocate %d bytes",sizeof(struct MFTPBVPNSpaceData));
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }
  data->n=nx*nu+np+nx;
  data->nx=nx;
  data->nu=nu;
  data->np=np;
  data->parmPeriodic=(int*)malloc(np*sizeof(int));
  if(data->parmPeriodic==(int*)NULL)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"Out of memory, trying to allocate %d bytes",np*sizeof(int));
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }
  data->parmPeriod=(double*)malloc(np*sizeof(double));
  if(data->parmPeriod==(double*)NULL)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"Out of memory, trying to allocate %d bytes",np*sizeof(double));
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }
  for(i=0;i<np;i++)
   {
    data->parmPeriodic[i]=0;
    data->parmPeriod[i]=0.;
   }
  
  MFNSpaceSetData(this,(void*)data);

  return this;
 }

void MFFreeTPBVPNSpaceData(void *data)
 {
  static char RoutineName[]={"MFFreeTPBVPNSpace"};
  struct MFTPBVPNSpaceData *d;

  d=(struct MFTPBVPNSpaceData*)data;
  if(d!=(struct MFTPBVPNSpaceData*)NULL)
   {
    if(d->parmPeriodic!=(int*)NULL)free(d->parmPeriodic);
    if(d->parmPeriod!=(double*)NULL)free(d->parmPeriod);
    free(d);
   }

  return;
 }

double MFTPBVPNSpaceDistance(MFNSpace this,MFNVector v0,MFNVector v1,void *d)
 {
  static char RoutineName[]={"MFNSpaceDistance"};
  double result;
  int i,j,o;
  int nx,nu,np;
  double t;
  double *u0;
  double *u1;
  struct MFTPBVPNSpaceData *data;

  data=(struct MFTPBVPNSpaceData*)d;
  nx=data->nx;
  nu=data->nu;
  np=data->np;

  result=0.;
  o=nx*nu+np;

  u0=MFNV_CStar(v0);
  u1=MFNV_CStar(v1);

  for(j=0;j<nu;j++)
    result+=(u1[j]-u0[j])*(u1[j]-u0[j])*u0[o]/2.;
  for(i=1;i<nx-1;i++)
   {
    for(j=0;j<nu;j++)
      result+=(u1[j+nu*i]-u0[j+nu*i])*(u1[j+nu*i]-u0[j+nu*i])*u0[o+i];
   }
  for(j=0;j<nu;j++)
    result+=(u1[j+(nx-1)*nu]-u0[j+(nx-1)*nu])*(u1[j+(nx-1)*nu]-u0[j+(nx-1)*nu])*u0[o+nx-1]/2.;

  for(i=0;i<np;i++)
   {
    if(!data->parmPeriodic[i])
     {
      result+=(u1[nx*nu+i]-u0[nx*nu+i])*(u1[nx*nu+i]-u0[nx*nu+i]);
     }else{
      t=fabs(u1[nx*nu+i]-u0[nx*nu+i]);
      if(t>data->parmPeriod[i]/2.)t=t-data->parmPeriod[i]/2.;
      result+=t*t;
     }
   }
  result=sqrt(result);

  return result;
 }

void MFTPBVPNSpaceDirection(MFNSpace this,MFNVector v0,MFNVector v1,MFNVector diff,void *d)
 {
  static char RoutineName[]={"MFNSpaceDirection"};
  int i;
  int nx,nu,np;
  struct MFTPBVPNSpaceData *data;
  double t;
  double *u0;
  double *u1;
  double *di;

  data=(struct MFTPBVPNSpaceData*)d;
  nx=data->nx;
  nu=data->nu;
  np=data->np;

  u0=MFNV_CStar(v0);
  u1=MFNV_CStar(v1);
  di=MFNV_CStar(diff);

  for(i=0;i<nx*nu;i++)di[i]=u1[i]-u0[i];

  for(i=0;i<np;i++)
   {
    if(!data->parmPeriodic[i])
     {
      di[nx*nu+i]=u1[nx*nu+i]-u0[nx*nu+i];
     }else{
      t=fabs(u1[nx*nu+i]-u0[nx*nu+i]);
      if(t<data->parmPeriod[i]/2.)
       {
        di[nx*nu+i]=u1[nx*nu+i]-u0[nx*nu+i];
       }else{
        di[nx*nu+i]=u0[nx*nu+i]-u1[nx*nu+i];
       }
     }
   }

  for(i=nx*nu+np;i<nx*nu+np+nx;i++)di[i]=u1[i];

  return;
 }

double MFTPBVPNSpaceInner(MFNSpace this,MFNVector v0,MFNVector v1,void *d)
 {
  static char RoutineName[]={"MFNSpaceInner"};
  double result;
  int i,j,o;
  int nx,nu,np,n;
  int verbose;
  struct MFTPBVPNSpaceData *data;
  double *u0;
  double *u1;

  data=(struct MFTPBVPNSpaceData*)d;
  n=data->n;
  nx=data->nx;
  nu=data->nu;
  np=data->np;

  verbose=0;

  if(verbose){printf("%s, n=%d, nx=%d, nu=%d, np=%d, n should be %d\n",RoutineName,n,nx,nu,np,nx*nu+np+nx);fflush(stdout);}

  data=(struct MFTPBVPNSpaceData*)d;
  u0=MFNV_CStar(v0);
  u1=MFNV_CStar(v1);

  o=nx*nu+np;
  result=0.;
  for(j=0;j<nu;j++)
    result+=u0[j]*u1[j]*u0[o]/2;
  for(i=1;i<nx-1;i++)
   {
    for(j=0;j<nu;j++)
      result+=u0[j+nu*i]*u1[j+nu*i]*u0[o+i];
   }
  for(j=0;j<nu;j++)
    result+=u0[j+(nx-1)*nu]*u1[j+(nx-1)*nu]*u0[o+nx-1]/2;

  o=nx*nu;
  for(i=0;i<np;i++)
    result+=u0[o+i]*u1[o+i];

  return result;
 }

void MFWriteTPBVPNSpaceData(FILE *fid,MFNSpace this,void *d)
 {
  static char RoutineName[]={"MFWriteTPBVPNSpaceData"};
  int i;
  struct MFTPBVPNSpaceData *data;

  data=(struct MFTPBVPNSpaceData*)d;

  fprintf(fid,"%d %d %d %d\n",data->n,data->nx,data->nu,data->np);

  return;
 }

void MFReadTPBVPNSpaceData(FILE *fid, MFNSpace this)
 {
  static char RoutineName[]={"MFTPBVPReadNSpaceData"};
  int i;
  struct MFTPBVPNSpaceData *data;

  data=(struct MFTPBVPNSpaceData*)malloc(sizeof(struct MFTPBVPNSpaceData));
  if(data==(struct MFTPBVPNSpaceData*)NULL)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"Out of memory, trying to allocate %d bytes",sizeof(struct MFTPBVPNSpaceData));
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }

  fscanf(fid,"%d %d %d %d\n",&(data->n),&(data->nx),&(data->nu),&(data->np));
  MFNSpaceSetData(this,data);

  return;
 }

void MFTPBVPNSpaceAdd(MFNSpace this,MFNVector v0,MFNVector v1,MFNVector sum,void *d)
 {
  static char RoutineName[]={"MFNSpaceAdd"};
  int i;
  int nx,nu,np;
  struct MFTPBVPNSpaceData *data;
  double *u0,*u1,*v;

  data=(struct MFTPBVPNSpaceData*)d;
  nx=data->nx;
  nu=data->nu;
  np=data->np;
  u0=MFNV_CStar(v0);
  u1=MFNV_CStar(v1);
  v =MFNV_CStar(sum);

  for(i=0;i<nx*nu+np;i++)
    v[i]=u1[i]+u0[i];
  for(i=nx*nu+np;i<nx*nu+np+nx;i++)
    v[i]=u1[i];

  return;
 }

void MFTPBVPNSpaceScale(MFNSpace this,double s, MFNVector v,MFNVector prod,void *d)
 {
  static char RoutineName[]={"MFNSpaceScale"};
  int i;
  int nx,nu,np;
  struct MFTPBVPNSpaceData *data;
  double *u,*p;

  if(this==(MFNSpace)NULL)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"Space (arg 1) is NULL!");
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }

  if(v==(MFNVector)NULL)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"v (arg 4) is NULL!");
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }

  if(prod==(MFNVector)NULL)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"prod (arg 5) is NULL!");
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }

  data=(struct MFTPBVPNSpaceData*)d;
  nx=data->nx;
  nu=data->nu;
  np=data->np;
  u=MFNV_CStar(v);
  p=MFNV_CStar(prod);

  for(i=0;i<nx*nu+np;i++)p[i]=s*u[i];
  for(i=nx*nu+np;i<nx*nu+np+nx;i++)p[i]=u[i];

  return;
 }

void MFTPBVPNSpaceSetPeriodicParameter(MFNSpace this,int p,double T)
 {
  struct MFTPBVPNSpaceData *data;
  static char RoutineName[]={"MFTPBVPNSpaceSetPeriodicParameter"};

  if(strcmp("TPBVPNSpace",MFNSpaceGetId(this)))
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"Space not a TPBVPNSpace!");
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }

  data=(struct MFTPBVPNSpaceData*)MFNSpaceGetData(this);

  if(p<0||p>=data->np)
   {
    sprintf(MFTPBVPNSpaceErrorMsg,"Parameter %d (argument 2) is invalid must be in range [0,%d]!",p,data->np);
    printf("%s -- %s\n",RoutineName,MFTPBVPNSpaceErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTPBVPNSpaceErrorMsg,__LINE__,__FILE__);
    return;
   }
  data->parmPeriodic[p]=1;
  data->parmPeriod[p]=T;

  return;
 }
