/* 
    @(#)MFTorus.c	1.12
    03/07/11 17:00:39
   
    PROGRAM NAME:  Manifold

    (c) COPYRIGHT INTERNATIONAL BUSINESS MACHINES
    CORPORATION 12/1/2001.  ALL RIGHTS RESERVED.

    Please refer to the LICENSE file in the top directory

*/
/*      author: Mike Henderson mhender@watson.ibm.com */
/*      date:   February 22, 1999                     */

#include <MFImplicitMF.h>
#include <MFError.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

static char MFTorusMFErrorMsg[256]="";
void MFSetError(int,char*,char*,int,char*);

void MFFreeTorusData(void*);
int MFProjectTorus(int,int,MFNVector,MFNKMatrix,MFNVector,void*,int*);
int MFTangentTorus(int,int,MFNVector,MFNKMatrix,void*);
double MFScaleTorus(int,int,MFNVector,MFNKMatrix,void*);
void MFWriteTorusData(FILE*,void*);
MFImplicitMF MFReadTorus(FILE*);

int MFTorusProjectToSave(MFNVector,double*,void*);
int MFTorusProjectToDraw(MFNVector,double*,void*);
int MFTorusProjectForBB(MFNVector,double*,void*);

MFImplicitMF MFIMFCreateTorus(double x,double y,double z,double RI, double RO)
 {
  static char RoutineName[]={"MFIMFCreateTorus"};
  MFImplicitMF torus;
  MFNSpace space;
  double *data;

  torus=MFIMFCreateBaseClass(3,2,"Torus");

  space=MFCreateNSpace(3);
  MFIMFSetSpace(torus,space);
  MFFreeNSpace(space);

  data=(double*)malloc(5*sizeof(double)); /*done*/
  if(data==(double*)NULL)
   {
    sprintf(MFTorusMFErrorMsg,"Out of memory, trying to allocate %d bytes",5*sizeof(double));
    printf("%s -- %s\n",RoutineName,MFTorusMFErrorMsg);fflush(stdout);
    MFSetError(12,RoutineName,MFTorusMFErrorMsg,__LINE__,__FILE__);
    free(torus);
    return (MFImplicitMF)NULL;
   }
  data[0]=x;
  data[1]=y;
  data[2]=z;
  data[3]=RI;
  data[4]=RO;
  MFIMFSetData(torus,data);
  MFIMFSetFreeData(torus,MFFreeTorusData);
  MFIMFSetProject(torus,MFProjectTorus);
  MFIMFSetTangent(torus,MFTangentTorus);
  MFIMFSetScale(torus,MFScaleTorus);
  MFIMFSetWriteData(torus,MFWriteTorusData);

  MFIMFSetProjectForSave(torus,MFTorusProjectToSave);
  MFIMFSetProjectForDraw(torus,MFTorusProjectToDraw);
  MFIMFSetProjectForBB(torus,MFTorusProjectForBB);

  return torus;
 }

void MFFreeTorusData(void *d)
 {
  static char RoutineName[]={"MFFreeTorusData"};
  free(d);
  return;
 }

#define MFTWOPI 6.2831853071795862320
double *MFKV_CStar(MFKVector);
double *MFNV_CStar(MFNVector);
double *MFNKM_CStar(MFNKMatrix);

int MFProjectTorus(int n,int k,MFNVector vu0,MFNKMatrix mPhi,MFNVector vu,void *d,int *index)
 {
  static char RoutineName[]={"MFProjectTorus"};
  double R,R1,R2;
  double nx,ny,nz;
  double x0,y0,z0;
  double x,y,z;
  double l0x,l0y,l0z;
  double l1x,l1y,l1z;
  double s,t,dt,F,dF;
  double disc,ddisc;
  double err;
  double *u0,*Phi,*u;

  u0=MFNV_CStar(vu0);
  Phi=MFNKM_CStar(mPhi);
  u=MFNV_CStar(vu);

/* (sqrt(x^2+y^2)-R1)^2+z^2 -R2^2 = 0; */

  l0x=Phi[0];
  l0y=Phi[1];
  l0z=Phi[2];
  l1x=Phi[0+3];
  l1y=Phi[1+3];
  l1z=Phi[2+3];
  nx=l0y*l1z-l0z*l1y;
  ny=l0z*l1x-l0x*l1z;
  nz=l0x*l1y-l0y*l1x;
  x0=u0[0]-((double*)d)[0];
  y0=u0[1]-((double*)d)[1];
  z0=u0[2]-((double*)d)[2];
  R1=((double*)d)[3];
  R2=((double*)d)[4];

  n=0;
  t=0.;
  while(TRUE)
   {
    disc=sqrt(pow(x0+t*nx,2)+pow(y0+t*ny,2));
    F=pow(disc-R1,2)+pow(z0+t*nz,2)-pow(R2,2);
    if(FALSE)printf(" iteration %d, t=%lf, (%lf,%lf,%lf), error=%lf\n",n,t,x,y,z,fabs(F));fflush(stdout);
    if(fabs(F)<1.e-7)
     {
      x=x0+t*nx;
      y=y0+t*ny;
      z=z0+t*nz;

      u[0]=((double*)d)[0]+x0+t*nx;
      u[1]=((double*)d)[1]+y0+t*ny;
      u[2]=((double*)d)[2]+z0+t*nz;
      if(FALSE)printf("\n");fflush(stdout);

      *index=0;
      return 1;
     }
    ddisc=(nx*(x0+t*nx)+ny*(y0+t*ny))/disc;
    dF=2*(disc-R1)*ddisc+2*(z0+t*nz)*nz;
    dt=-F/dF;
    if(dt>.01)dt=.01;
    if(dt<-.01)dt=-.01;
    if(n>20)
     {
/*    printf(" too many iterations, failing.");fflush(stdout);abort();*/
      return 0;
     }
    t=t+dt;
    n++;
   }
  *index=0;
  return 1;
 }

int MFTangentTorus(int n,int k,MFNVector vu,MFNKMatrix mPhi,void *d)
 {
  static char RoutineName[]={"MFTangentTorus"};
  double D,s;
  double x,y,z;
  double R1,R2;
  double *u,*Phi;

  Phi=MFNKM_CStar(mPhi);
  u=MFNV_CStar(vu);

/* (sqrt(x^2+y^2)-R1)^2+z^2 -R2^2 = 0; */

  x=u[0]-((double*)d)[0];
  y=u[1]-((double*)d)[1];
  z=u[2]-((double*)d)[2];
  R1=((double*)d)[3];
  R2=((double*)d)[4];

/*printf("Torus Tangent (%lf,%lf,%lf)\n",x,y,z);fflush(stdout);*/

/* (1-R1/sqrt(x^2+y^2)) (2x x' + 2y y') +2z z' = 0 */

  D=1-R1/sqrt(x*x+y*y);
  Phi[0]= y*D;
  Phi[1]=-x*D;
  Phi[2]=0.;
  s=1./sqrt(Phi[0]*Phi[0]+Phi[1]*Phi[1]);
  Phi[0]=Phi[0]*s;
  Phi[1]=Phi[1]*s;

/*printf("      Tangent 1 (%lf,%lf,%lf) err=%lf nrm=%lf\n",Phi[0],Phi[1],Phi[2],
    (1.-R1/sqrt(x*x+y*y))*(x*Phi[0]+y*Phi[1])+z*Phi[2],
    Phi[0]*Phi[0]+Phi[1]*Phi[1]+Phi[2]*Phi[2]
    );fflush(stdout);
*/

  if(fabs(y)>.1)
   {
    Phi[4]=-y*z;
    Phi[5]=D*(x*x+y*y);
    Phi[3]=x/y*Phi[4];
   }else{
    Phi[3]=-x*z;
    Phi[5]=D*(x*x+y*y);
    Phi[4]=y/x*Phi[3];
   }

  s=1./sqrt(Phi[3]*Phi[3]+Phi[4]*Phi[4]+Phi[5]*Phi[5]);
  Phi[3]=Phi[3]*s;
  Phi[4]=Phi[4]*s;
  Phi[5]=Phi[5]*s;

/*printf("      Tangent 2 (%lf,%lf,%lf) err %lf  ortho? %lf norm %lf\n",Phi[3],Phi[4],Phi[5],
    (1-R1/sqrt(x*x+y*y))*(x*Phi[3]+y*Phi[4])+z*Phi[5],
    Phi[0]*Phi[3]+Phi[1]*Phi[4]+Phi[2]*Phi[5],
    Phi[3]*Phi[3]+Phi[4]*Phi[4]+Phi[5]*Phi[5]
    );fflush(stdout);*/

  return 1;
 }

extern double MFEpsilon;

double MFScaleTorus(int n,int k,MFNVector vu,MFNKMatrix mPhi,void *d)
 {
  static char RoutineName[]={"MFScaleTorus"};

/* (sqrt(x^2+y^2)-R1)^2+z^2 -R2^2 = 0; */
/* (1-R1/sqrt(x^2+y^2)) (x*dx + y*dy) +z dz = 0 */
/* (1-R1/sqrt(x^2+y^2)) (x*dDx +y*dDy)+z*dDz+(1-R1/sqrt(x^2+y^2)) (dxDx+dyDy)+dzDz+R1/sqrt(x^2+y^2)(xDx+yDy)(xdx+ydy) +dzDz = 0 */

  double dg,ddg;
  double sqrtR;
  double x,y,z,R1,R2;
  double nx,ny,nz;
  double l,r;
  double dx,dy,dz;
  double Dx,Dy,Dz;
  double d00,d01,d11;
  double *u,*Phi;

  Phi=MFNKM_CStar(mPhi);
  u=MFNV_CStar(vu);

  x=u[0]-((double*)d)[0];
  y=u[1]-((double*)d)[1];
  z=u[2]-((double*)d)[2];
  R1=((double*)d)[3];
  R2=((double*)d)[4];
  sqrtR=sqrt(x*x+y*y);

  nx=Phi[1]*Phi[5]-Phi[2]*Phi[4];
  ny=Phi[2]*Phi[3]-Phi[0]*Phi[5];
  nz=Phi[0]*Phi[4]-Phi[1]*Phi[3];

  dx=Phi[0];
  dy=Phi[1];
  dz=Phi[2];
  Dx=Phi[0];
  Dy=Phi[1];
  Dz=Phi[2];
  ddg=(1-R1/sqrtR)*(dx*Dx+dy*Dy)+dz*Dz
      +R1/sqrtR/(x*x+y*y)*(x*Dx+y*Dy)*(x*dx+y*dy)+dz*Dz;
  dg=(1-R1/sqrtR)*(x*nx+y*ny)+z*nz;
  d00=-ddg/dg;

  dx=Phi[0];
  dy=Phi[1];
  dz=Phi[2];
  Dx=Phi[3];
  Dy=Phi[4];
  Dz=Phi[5];
  ddg=(1-R1/sqrtR)*(dx*Dx+dy*Dy)+dz*Dz
      +R1/sqrtR/(x*x+y*y)*(x*Dx+y*Dy)*(x*dx+y*dy)+dz*Dz;
  dg=(1-R1/sqrtR)*(x*nx+y*ny)+z*nz;
  d01=-ddg/dg;

  dx=Phi[3];
  dy=Phi[4];
  dz=Phi[5];
  Dx=Phi[3];
  Dy=Phi[4];
  Dz=Phi[5];
  ddg=(1-R1/sqrtR)*(dx*Dx+dy*Dy)+dz*Dz
      +R1/sqrtR/(x*x+y*y)*(x*Dx+y*Dy)*(x*dx+y*dy)+dz*Dz;
  dg=(1-R1/sqrtR)*(x*nx+y*ny)+z*nz;
  d11=-ddg/dg;

/* Ev sat (d00-l)*(d11-l)+d01*d01=0 */

  if(d00+d11>0)
    l=.5*(d00+d11)+.5*sqrt((d00-d11)*(d00-d11)+4*d01*d01);
   else
    l=-.5*(d00+d11)+.5*sqrt((d00-d11)*(d00-d11)+4*d01*d01);

  r=sqrt(2*MFEpsilon/l);

/*
  printf("  MFScaleTorus, (%lf,%lf,%lf), R1=%lf, R2=%lf\n",x,y,z,R1,R2);fflush(stdout);
  printf("                c0=%lf c1=%lf, lev=%lf, r=%lf\n",d00,d11,l,r);fflush(stdout);
*/

  return r;
 }

void MFWriteTorusData(FILE *fid,void *d)
 {
  static char RoutineName[]={"MFWriteTorusData"};
  double *data;

  data=(double*)d;

  fprintf(fid,"%lf %lf %lf %lf %lf\n",data[0],data[1],data[2],data[3],data[4]);
  fflush(fid);
  return;
 }

MFImplicitMF MFReadTorus(FILE *fid)
 {
  static char RoutineName[]={"MFReadTorus"};
  MFImplicitMF torus;
  double x=0.;
  double y=0.;
  double z=0.;
  double RI=0.;
  double RO=0.;

  fscanf(fid,"%lf %lf %lf %lf %lf\n",&x,&y,&z,&RI,&RO);
  torus=MFIMFCreateTorus(x,y,z,RI,RO);

  return torus;
 }

int MFTorusProjectToSave(MFNVector u, double *x, void *d)
 {
  if(x==(double*)NULL)return 3;

  x[0]=MFNV_C(u,0);
  x[1]=MFNV_C(u,1);
  x[2]=MFNV_C(u,2);

  return 0;
 }

int MFTorusProjectToDraw(MFNVector u, double *x, void *d)
 {
  if(x==(double*)NULL)return 3;

  x[0]=MFNV_C(u,0);
  x[1]=MFNV_C(u,1);
  x[2]=MFNV_C(u,2);

  return 0;
 }

int MFTorusProjectForBB(MFNVector u, double *x, void *d)
 {
  if(x==(double*)NULL)return 3;

  x[0]=MFNV_C(u,0);
  x[1]=MFNV_C(u,1);
  x[2]=MFNV_C(u,2);

  return 0;
 }
